@@ -1237,3 +1237,89 @@ int write_blocked(const bdaddr_t *local, const bdaddr_t *remote,
 
 	return textfile_caseput(filename, addr, "");
 }
+
+int write_device_services(const bdaddr_t *sba, const bdaddr_t *dba,
+							const char *services)
+{
+       char filename[PATH_MAX + 1], addr[18];
+
+       create_filename(filename, PATH_MAX, sba, "primary");
+
+       create_file(filename, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
+
+       ba2str(dba, addr);
+
+       return textfile_put(filename, addr, services);
+}
+
+char *read_device_services(const bdaddr_t *sba, const bdaddr_t *dba)
+{
+       char filename[PATH_MAX + 1], addr[18];
+
+       create_filename(filename, PATH_MAX, sba, "primary");
+
+       create_file(filename, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
+
+       ba2str(dba, addr);
+
+       return textfile_caseget(filename, addr);
+}
+
+int write_device_characteristics(const bdaddr_t *sba, const bdaddr_t *dba,
+					uint16_t handle, const char *chars)
+{
+	char filename[PATH_MAX + 1], addr[18], key[23];
+
+	create_filename(filename, PATH_MAX, sba, "characteristic");
+
+	create_file(filename, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
+
+	ba2str(dba, addr);
+
+	snprintf(key, sizeof(key), "%17s#%04X", addr, handle);
+
+	return textfile_put(filename, key, chars);
+}
+
+char *read_device_characteristics(const bdaddr_t *sba, const bdaddr_t *dba,
+							uint16_t handle)
+{
+	char filename[PATH_MAX + 1], addr[18], key[23];
+
+	create_filename(filename, PATH_MAX, sba, "characteristic");
+
+	create_file(filename, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
+
+	ba2str(dba, addr);
+
+	snprintf(key, sizeof(key), "%17s#%04X", addr, handle);
+
+	return textfile_caseget(filename, key);
+}
+
+int write_device_attribute(const bdaddr_t *sba, const bdaddr_t *dba,
+					uint16_t handle, const char *chars)
+{
+	char filename[PATH_MAX + 1], addr[18], key[23];
+
+	create_filename(filename, PATH_MAX, sba, "attributes");
+
+	create_file(filename, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
+
+	ba2str(dba, addr);
+
+	snprintf(key, sizeof(key), "%17s#%04X", addr, handle);
+
+	return textfile_put(filename, key, chars);
+}
+
+int read_device_attributes(const bdaddr_t *sba, textfile_cb func, void *data)
+{
+	char filename[PATH_MAX + 1];
+
+	create_filename(filename, PATH_MAX, sba, "attributes");
+
+	create_file(filename, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
+
+	return textfile_foreach(filename, func, data);
+}
