@@ -2,7 +2,7 @@
  *
  *  BlueZ - Bluetooth protocol stack for Linux
  *
- *  Copyright (C) 2004-2009  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2004-2010  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -409,7 +409,8 @@ static int cmd_buildname(int transport, int argc, char *argv[])
 {
 	uint8_t array[130];
 	char name[64];
-	int i, err;
+	unsigned int i;
+	int err;
 
 	OPT_HELP(0, NULL);
 
@@ -699,7 +700,7 @@ static int cmd_psget(int transport, int argc, char *argv[])
 		return err;
 
 	length = array[2] + (array[3] << 8);
-	if (length + 6 > sizeof(array) / 2)
+	if (length + 6 > (int) sizeof(array) / 2)
 		return -EIO;
 
 	memset(array, 0, sizeof(array));
@@ -774,7 +775,7 @@ static int cmd_psset(int transport, int argc, char *argv[])
 		return err;
 
 	length = array[2] + (array[3] << 8);
-	if (length + 6 > sizeof(array) / 2)
+	if (length + 6 > (int) sizeof(array) / 2)
 		return -EIO;
 
 	memset(array, 0, sizeof(array));
@@ -962,7 +963,7 @@ static int cmd_psread(int transport, int argc, char *argv[])
 			continue;
 
 		length = array[2] + (array[3] << 8);
-		if (length + 6 > sizeof(array) / 2)
+		if (length + 6 > (int) sizeof(array) / 2)
 			continue;
 
 		memset(array, 0, sizeof(array));
@@ -983,7 +984,7 @@ static int cmd_psread(int transport, int argc, char *argv[])
 			str = NULL;
 		}
 
-		printf("// %s%s\n&%04x =", str ? "PSKEY_" : "", 
+		printf("// %s%s\n&%04x =", str ? "PSKEY_" : "",
 						str ? str : val, pskey);
 		for (i = 0; i < length; i++)
 			printf(" %02x%02x", array[(i * 2) + 7], array[(i * 2) + 6]);
